(***********************************************************************

                    Mathematica-Compatible Notebook

This notebook can be used on any computer system with Mathematica 4.0,
MathReader 4.0, or any compatible application. The data for the notebook 
starts with the line containing stars above.

To get the notebook into a Mathematica-compatible application, do one of 
the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing the 
word CacheID, otherwise Mathematica-compatible applications may try to 
use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
***********************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[     22530,        454]*)
(*NotebookOutlinePosition[     23408,        482]*)
(*  CellTagsIndexPosition[     23364,        478]*)
(*WindowFrame->Normal*)



Notebook[{
Cell[BoxData[{
    StyleBox[\(\(\n\)\( (*\ \(\(:\)\(Title : \ 
            Decomposing\ Matrices\ into\ Elementary\ Matrices\)\), \ 
        Permutation\ Matrices\ and\ a\ Diagonal\ \(\(Matrix\)\(.\)\)*) \)\(\n\
\)\(\n\)\( (*\ \(:\)\(Author : \ Frank\ Zizza\n
          \t\ \ \ \ \ \ \ \ Willamette\ University\n
          \t\ \ \ \ \ \ \ \ zizza@willamette . 
            edu\)\n*) \)\(\n\)\(\n\)\( (*\ \(\(:\)\(Summary : \n\ \ \ \
Decomposes\ matrices\ with\ entries\ in\ a\ Euclidean\ ring\ as\ a\ product\ \
of\ elementary\ matrices\)\), \ 
        permutation\ matrices\ and\ a\ diagonal\ matrix . \ \ Allowed\ \
Euclidean\ rings\ are\ the\ integers, \ the\ Gaussian\ integers, \ 
        and\ the\ polynomials\ in\ one\ variable\ over\ the\ fields\ of\ \
rational\ numbers, \ 
        real\ numbers\ or\ complex\ \(\(numbers\)\(.\)\)\ \n*) \)\(\n\)\(\n\)\
\( (*\ \(:\)\(Context : \ 
            ElementaryDecompositions`\)\ *) \)\(\n\)\(\n\)\( (*\ \
\(:\)\(Package\ \(Version : \ 
              1.0\)\)\ *) \)\(\n\)\(\n\)\( (*\ \(:\)\(Copyright : \ 
            Copyright\ 1992\)\ *) \)\(\n\)\(\n\)\( (*\ \(\(:\)\(History : \n\ \
\ Created\ by\ Frank\ Zizza\ at\ MSRI\)\), \ 
        November\ 1992\n*) \)\(\n\)\(\n\)\( (*\ \(\(:\)\(Keywords : \ \n\ \ \
Euclidean\ ring\)\), \ elementary\ matrices, \ permutation\ matrices, \ 
        invariant\ factors, \ 
        elementary\ \(\(divisors\)\(.\)\)\n*) \)\(\n\)\(\n\)\( (*\ \
\(\(:\)\(Source : \n\ \ \(Jacobson : \ "\<Elementary Algebra I\>"\)\)\), \ \
\(W . \ H . \ 
              Freeman\  &\)\ \(\(Company\)\(.\)\)\ \n*) \)\(\n\)\(\n\)\(\n\
\)\( (*\ \(:\)\(Mathematica\ \(Version : \ 
              2.1\)\)*) \)\(\n\)\(\n\)\( (*\ \(:\)\(Limitations : \ 
            None\ \(\(known\)\(.\)\)\)\ *) \)\(\n\)\(\n\)\(\n\)\(\n\
\)\(BeginPackage["\<ElementaryDecompositions`\>"]\)\(\n\)\(\n\)\(\n\)\( \
(*Usage*) \)\(\n\)\),
      FontFamily->"Courier New",
      FontSize->16], "\n", 
    StyleBox[\(\(ElementaryDecomposition::usage\  = \ \
"\<ElementaryDecomposition[M] yields the factorization of the matrix M into \
elementary matrices, permutation matrices and a diagonal matrix.  The matrix \
M must have entries  in the ring of integers, or the ring of Gaussian \
integers. The result is a list {{e[i, j, a], p[i, j] ...}, {{{d[1],0,..}, {0, \
d[2], 0, ..}}}, {e[i, j, a], p[i, j] ...}}.  e[i, j, x] is the identity \
matrix except in position (i, j) where is has the value x. p[i, j] is the \
identity matrix with the i-th and j-th columns interchanged. The middle term \
in the result is a diagonal matrix {{d[1],0,..}, {0, d[2], 0, ..}}. The \
orginal matrix M is the product of all the matrices in the list in the order \
in which they are printed.
\n \nElementaryDecomposition[M, t] yields the factorization for matrices with \
entries in the ring of polynomials in one variable t over a field.  The field \
is determined by the coefficients of the polynomials in the input matrix.
\n \nElementaryDecomposition[M, Padded -> True] yields the same factorization \
as ElementaryDecomposition[M] with the elementary matrices and permutation \
matrices expanded as usual Mathematica matrices.\>"\)\(\n\)\),
      FontFamily->"Courier New",
      FontSize->16], "\n", 
    StyleBox[\(\(NormalDecomposition::usage\  = \ "\<NormalDecomposition[M] \
yields a factorization of the r x s matrix M into {P, D, Q} where P is r x r \
and invertable, D is r x s and diagonal and Q is s x s and invertable.  The \
original matrix M equals P . D . Q.  The entries of M must all be integers or \
Gaussian integers.  NormalDecomposition[M, t] works polynomials in a single \
variable over a field.\>"\)\(\n\)\),
      FontFamily->"Courier New",
      FontSize->16], "\n", 
    StyleBox[\(\(InvariantFactors::usage\  = \ "\<InvariantFactors[M] gives \
the diagonal elements of the normal form of the matrix M under the \
equivalence relation A ~ B iff A == P . B . Q where P and Q are invertable \
matrices.  The entries of M must all be integers or Gaussian \
integers.InvariantFactors[M, t] works for polynomials in a single variable \
over a field.  It is a standard theorem that all such matrices have a \
diagonal normal form.  The result is a list of elements {d[1], d[2], ... , \
d[k], 0, .. 0} of the appropriate ring with the divisibility condition d[i] | \
d[j] for all 1 <= i <= j <= k.\>"\)\(\n\)\),
      FontFamily->"Courier New",
      FontSize->16], "\n", 
    StyleBox[\(\(ElementaryMatrix::usage\  = \ "\<ElementaryMatrix[i, j, x, \
n] is a short form for n x n matrix which equals the identity matrix except \
in position (i, j) where is has the value x. In formatted output, the \
dimension n is suppressed and i and j are subscripted.\>"\)\(\n\)\),
      FontFamily->"Courier New",
      FontSize->16], "\n", 
    StyleBox[\(\(PermutationMatrix::usage\  = \ "\<PermutationMatrix[i, j, n] \
is a short form for n x n matrix that equals the identity matrix with the \
i-th and j-th columns interchanged. In formatted output, the dimension n is \
suppressed and i and j are subscripted.\>"\)\(\n\)\),
      FontFamily->"Courier New",
      FontSize->16], "\n", 
    StyleBox[\(\(Pad::usage\  = \ "\<Pad[elementary or permutation matrix] \
expands ElementaryMatrix[i, j, x, n] and PermutationMatrix[i, j, n] as actual \
matrices, i.e. a list of lists.\>"\)\(\n\)\),
      FontFamily->"Courier New",
      FontSize->16], "\n", 
    StyleBox[\(\(Padded::usage\  = \ "\<Padded is an option for \
ElementaryDecomposition which specfies whether the elementary matrices and \
the permutation matrices in the factorization should be given in compressed \
form as e[i,j,a] and p[i,j] or as actual matrices.  Padded -> True gives the \
actual matrices.  The default is Padded -> False.\>"\)\(\n\)\),
      FontFamily->"Courier New",
      FontSize->16], "\n", 
    StyleBox[\(\(CheckIt::usage\  = \ "\<CheckIt[factorization] multiplies \
the list of matrices obtained from ElementaryDecomposition.\>"\)\(\n\)\(\n\
\)\(\n\)\( (*Options*) \)\(\n\)\),
      FontFamily->"Courier New",
      FontSize->16], "\n", 
    StyleBox[\(\(Options[
            ElementaryDecomposition]\  = \ {Padded\  -> \ 
              False};\)\(\n\)\(\n\)\(\n\)\( (*Messages*) \)\(\n\)\),
      FontFamily->"Courier New",
      FontSize->16], "\n", 
    StyleBox[\(\(ElementaryDecomposition::notint\  = \ "\<Matrix does not \
have entries in the integers or the Gaussian integers. For polynomial \
entries, append the variable as in ElementaryDecomposition[M, t].\>"\)\(\n\
\)\),
      FontFamily->"Courier New",
      FontSize->16], "\n", 
    StyleBox[\(\(ElementaryDecomposition::notpoly\  = \ "\<Matrix does not \
have entries in the ring of polynomials in one variable.\>"\)\(\n\)\(\n\)\),
      FontFamily->"Courier New",
      FontSize->16], "\n", 
    StyleBox[\(\(Begin["\<`Private`\>"]\)\(\n\)\(\n\)\(\n\)\),
      FontFamily->"Courier New",
      FontSize->16], "\n", 
    StyleBox[\(\(SetAttributes[{LeftFactors, \ 
          RightFactors}, \ {HoldAll}]\)\(\n\)\),
      FontFamily->"Courier New",
      FontSize->16], "\n", 
    StyleBox[\(\(MakeFactorList[A_]\  := \ 
        Factors[LeftFactors[], \ A, \ 
          RightFactors[]]\)\(\n\)\(\n\)\(\n\)\( (*Matrix\ Operations*) \)\(\n\
\)\(\n\)\),
      FontFamily->"Courier New",
      FontSize->16], "\n", 
    StyleBox[\(\(FindMinPosition[A_?MatrixQ]\  := \ \n\t
        Module[{B, \ min}, \n\t\tB\  = \ $Delta[\ A\ ]; \n\t\tmin\  = \ 
            Min[\ Flatten[B]\ \ ]; \n\t\tPosition[\ B, \ min, \ 2, \ 
            1]]\)\(\n\)\(\t\t\)\),
      FontFamily->"Courier New",
      FontSize->16], "\n", 
    StyleBox[\(\(SubMatrix[A_?MatrixQ, \ n_Integer]\  := \ 
        Map[Drop[#, \ n - 1] &, \ Drop[A, \ n - 1]]\)\(\n\)\),
      FontFamily->"Courier New",
      FontSize->16], "\n", 
    StyleBox[\(\(FindMinPosition[A_?MatrixQ, \ n_Integer]\  := \ \n\t
        FindMinPosition[\ SubMatrix[A, \ n]\ ]\  + \ \((n - 1)\)\)\(\n\)\),
      FontFamily->"Courier New",
      FontSize->16], "\n", 
    StyleBox[\(\(SwitchRows[{i_, \ i_}, \ fac_Factors]\  := \ fac\)\(\n\)\),
      FontFamily->"Courier New",
      FontSize->16], "\n", 
    StyleBox[\(\(SwitchColumns[{i_, \ i_}, \ fac_Factors]\  := \ 
        fac\)\(\n\)\),
      FontFamily->"Courier New",
      FontSize->16], "\n", 
    StyleBox[\(\(SwitchRows[{i_, \ j_}, \ A_?MatrixQ]\  := \ \n\t
        A[\([\ \ Range[\ \(Dimensions[A]\)[\([1]\)]\ ]\ \  /. \ {i\  -> \ 
                  j, \ j\  -> \ i}\ \ ]\)]\)\(\n\)\),
      FontFamily->"Courier New",
      FontSize->16], "\n", 
    StyleBox[\(\(SwitchRows[{i_, \ j_}, \ 
          Factors[llist_, \ A_, \ rlist_]]\  := \n\t
        Module[{m}, \n\tm\  = \ \(Dimensions[A]\)[\([1]\)]; \n\t
          Factors[Append[llist, PermutationMatrix[i, j, m]], 
            SwitchRows[{i, j}, \ A], \ rlist]\ ]\)\(\n\)\),
      FontFamily->"Courier New",
      FontSize->16], "\n", 
    StyleBox[\(\(SwitchColumns[{i_, \ i_}, \ A_?MatrixQ]\  := \ A\)\(\n\)\),
      FontFamily->"Courier New",
      FontSize->16], "\n", 
    StyleBox[\(\(SwitchColumns[{i_, \ j_}, \ A_?MatrixQ]\  := \ \n\t
        Transpose[\ SwitchRows[{i, \ j}, \ Transpose[A]\ ]\ ]\)\(\n\)\),
      FontFamily->"Courier New",
      FontSize->16], "\n", 
    StyleBox[\(\(SwitchColumns[{i_, \ j_}, \ 
          Factors[llist_, \ A_, \ rlist_]]\  := \n\t
        Module[{n}, \n\t
          n\  = \ \(Dimensions[A]\)[\([2]\)]; \n\t\ Factors[llist, \ 
            SwitchColumns[{i, \ j}, \ A], \ 
            Prepend[rlist, \ PermutationMatrix[i, j, n]]\ ]\ ]\)\(\n\)\),
      FontFamily->"Courier New",
      FontSize->16], "\n", 
    StyleBox[\(\(Reposition[fac : Factors[llist_, \ A_, \ rlist_], \ 
          n_Integer]\  := \ \n\t
        Module[{i, \ j}, \n\t\t{{i, \ j}}\  = \ 
            FindMinPosition[A, \ n]; \n\t\tSwitchRows[\ {n, \ 
              i}, \ \ SwitchColumns[\ {n, \ j}, \ \ fac]\ ]\ ]\)\(\n\)\),
      FontFamily->"Courier New",
      FontSize->16], "\n", 
    StyleBox[\(\(RowOperations[fac : Factors[llist_, \ A_, \ rlist_], \ 
          n_Integer]\  := \ \n\t
        Module[{ann\  = \ A[\([n, \ n]\)], \ B\  = \ A, \ m, \ an, \ i, \ 
            q, \ nllist\  = \ llist}, \n\t\ \ \ \ \ \ If[ann\  === \ 0, \ 
            fac, \n\t\tm\  = \ \(Dimensions[A]\)[\([1]\)]; \n\t\tFor[
              i = n + 1; \ an\  = \ A[\([n]\)], \ 
              i <= m, \ \(i++\), \n\t\t\ \ \ \ \ \ If[\((q\  = \ $Quotient[\ 
                        A[\([i, n]\)], \ ann])\)\  =!= \ 
                  0, \ \n\t\t\t\ \ AppendTo[nllist, \ 
                  ElementaryMatrix[i, \ n, \ q, \ 
                    m]]; \ \n\t\t\t\ \ \t\t\ \ \ B[\([i]\)]\  = \ 
                  Chop[Simplify[B[\([i]\)]\  - \ q*an]]]]; \n\t\tFactors[
              nllist, \ \ \ \ B, \ \ \ \ rlist]\ ]\ ]\)\(\n\)\),
      FontFamily->"Courier New",
      FontSize->16], "\n", 
    StyleBox[\(\(ColumnOperations[fac : Factors[llist_, \ A_, \ rlist_], \ 
          n_Integer]\  := \ \n\t
        Module[{ann\  = \ A[\([n, \ n]\)], \ B\  = \ Transpose[A], \ m, \ 
            an, \ i, \ q, \ nrlist\  = \ rlist}, \n\t\ \ \ \ \ \ If[
            ann\  === \ 0, \ 
            fac, \n\t\tm\  = \ \(Dimensions[A]\)[\([2]\)]; \n\t\tFor[
              i = n + 1; \ an\  = \ B[\([n]\)], \ 
              i <= m, \ \(i++\), \n\t\t\ \ \ \ \ \ If[\((q\  = \ $Quotient[
                        B[\([i, n]\)], \ ann])\)\  =!= \ 
                  0, \ \n\t\t\t\ \ PrependTo[nrlist, \ 
                  ElementaryMatrix[n, \ i, \ q, \ 
                    m]]; \ \n\t\t\t\ \ \t\t\ \ \ \ B[\([i]\)]\  = \ 
                  Chop[Simplify[B[\([i]\)]\  - \ q*an]]]]; \n\t\tFactors[
              llist, \ \ \ \ Transpose[B], \ \ \ \ nrlist]\ ]\ ]\)\(\n\)\),
      FontFamily->"Courier New",
      FontSize->16], "\n", 
    StyleBox[\(\(RCOperations[fac_Factors, \ n_Integer]\  := \ \n\t
        ColumnOperations[\ RowOperations[\ Reposition[fac, \ n], \ n], \ 
          n]\)\(\n\)\(\n\)\),
      FontFamily->"Courier New",
      FontSize->16], "\n", 
    StyleBox[\(\(OneStep[fac_Factors, \ n_Integer]\  := \ 
        FixedPoint[RCOperations[#, \ n] &, \ fac]\)\(\n\)\),
      FontFamily->"Courier New",
      FontSize->16], "\n", 
    StyleBox[\(\(DivisibleQ[A_?MatrixQ]\  := \ \n\t
        Module[{a}, \ \n\t\tIf[\ \((a\  = \ 
                  Simplify[A[\([1, 1]\)]])\)\  =!= \ 
              0, \ \n\t\tUnion[\ $Mod[\ Flatten[A], \ 
                  a\ ]]\ \  === \ {0}, \ \n\t\tTrue]\ ]\)\(\n\)\),
      FontFamily->"Courier New",
      FontSize->16], "\n", 
    StyleBox[\(\(AddRow[{i_, \ j_}, \ 
          Factors[llist_, \ A_, \ rlist_]]\  := \ \n\ \ \ \ \ Module[{m}, \n\t
          m\  = \ \(Dimensions[A]\)[\([1]\)]; \n\t
          Factors[Append[llist, \ 
              ElementaryMatrix[i, \ j, \ \(-1\), \ m]], \ \n\t\t\tReplacePart[
              A, \ A[\([i]\)] + A[\([j]\)], \ i], \ rlist]]\)\(\n\)\),
      FontFamily->"Courier New",
      FontSize->16], "\n", 
    StyleBox[\(\(DivisibilityReposition[
          fac : Factors[llist_, \ A_, \ rlist_], \ n_]\  := \ \n\t
        Module[{B, \ i, \ u}, \ \n\t\tB\  = \ SubMatrix[A, \ n]; \n\t\tIf[\ 
            DivisibleQ[B], \ \n\t\t\tfac, \ \n\t\t\t{u}\  = \ 
              Select[\ Flatten[
                  B], \ $Mod[#, \ B[\([1, 1]\)]\ ]\  =!= \ 0 &, \ 
                1]; \n\t\t\ti\  = \ \(Position[\ B, \ u, \ 2, \ 1]\)[\([1, 
                  1]\)]; \n\t\t\tAddRow[{n, \ n + i - 1}, \ 
              fac]\ ]]\)\(\n\)\),
      FontFamily->"Courier New",
      FontSize->16], "\n", 
    StyleBox[\(\(DivisibleOneStep[fac_Factors, \ n_]\  := \ \n\ \ FixedPoint[
          DivisibilityReposition[OneStep[#1, \ n], \ n]\  &, \ fac]\)\(\n\)\),
      
      FontFamily->"Courier New",
      FontSize->16], "\n", 
    StyleBox[\(\(Diagonalize[\((A_)\)?MatrixQ]\  := \ \n\ \ Fold[
          DivisibleOneStep, \ MakeFactorList[A], \ 
          Range[Min[
              Dimensions[
                A]]]]\)\(\n\)\(\n\)\(\n\)\(\n\)\( (*User\ Interface*) \)\(\n\
\)\),
      FontFamily->"Courier New",
      FontSize->16], "\n", 
    StyleBox[\(\(GaussianIntegerQ[z_]\  := \ 
          MatchQ[z, \ 
            Complex[_Integer, \ _Integer]\  | \ _Integer];\)\(\n\)\),
      FontFamily->"Courier New",
      FontSize->16], "\n", 
    StyleBox[\(\(ElementaryDecomposition[matrix_?MatrixQ, \ 
          opts___Rule]\  := \ \n\t
        Module[{ans}, \n\t\ \ \ \ Which[\n\t\tMatrixQ[matrix, \ 
              IntegerQ], \n\t\t\t$Delta\  = \ 
              Function[n, \ 
                If[n =!= 0, \ Abs[n], \ 
                  Infinity], \ {Listable}]; \ \n\t\t\t$Quotient\  = \ 
              Function[{n, \ m}, \ Quotient[n, m]]; \n\t\t\t$Mod\  = \ 
              Function[{n, \ m}, \ Mod[n, m], \ {Listable}];, \n\t\tMatrixQ[
              matrix, \ GaussianIntegerQ], \n\t\t\t$Delta\  = \ 
              Function[n, \ 
                If[n =!= 0, \ Abs[n]^2, \ 
                  Infinity], \ {Listable}]; \n\t\t\t$Quotient\  = \ 
              Function[{n, \ m}, \ Round[n\ /\ m]]; \n\t\t\t$Mod\  = \ 
              Function[{n, \ m}, \ 
                n\  - \ Round[n\ /\ m]\ *\ 
                    m, \ {Listable}];, \n\t\tTrue, \n\t\t\tMessage[
              ElementaryDecomposition::notint]; \ 
            Abort[]]; \n\t\ \ \ \ \ ans\  = \ 
            Diagonalize[matrix]; \ \n\t\ \ \ \ \ ans\  = \ 
            Apply[List, \ 
              MapAt[List, \ ans, \ 2], \ {0, \ 
                1}]; \n\t\ \ \ \ \ padded\  = \ \(Padded\  /. \ {opts}\)\  \
/. \ Options[ElementaryDecomposition]; \n\t\ \ \ \ \ If[
            padded\  === \ False, \ ans, \ 
            Pad[ans]]]\)\(\n\)\(\t\ \ \ \ \ \)\(\n\)\),
      FontFamily->"Courier New",
      FontSize->16], "\n", 
    StyleBox[\(\(ElementaryDecomposition[matrix_?MatrixQ, \ t_, \ 
          opts___Rule]\  := \ \n\t
        Module[{ans}, \n\t\ \ \ \ If[
            Not[MatrixQ[matrix, \ PolynomialQ[#, \ t] &]], \ 
            Message[ElementaryDecomposition::notpoly]; \ 
            Abort[]]; \n\t\ \ \ \ $Delta\  = \ 
            Function[p, \ 
              If[p =!= 0, \ Exponent[p, \ t], \ 
                Infinity], \ {Listable}]; \n\t\ \ \ \ $Quotient\  = \ 
            Function[{p, \ q}, \ 
              PolynomialQuotient[p, \ q, \ t]]; \n\t\ \ \ \ $Mod\  = \ 
            Function[{p, q}, \ 
              PolynomialRemainder[p, \ q, \ 
                t], \ {Listable}]; \n\t\ \ \ \ \ ans\  = \ 
            Diagonalize[matrix]; \ \n\t\ \ \ \ \ ans\  = \ 
            Apply[List, \ 
              MapAt[List, \ ans, \ 2], \ {0, \ 
                1}]; \n\t\ \ \ \ \ padded\  = \ \(Padded\  /. \ {opts}\)\  \
/. \ Options[ElementaryDecomposition]; \n\t\ \ \ \ \ If[
            padded\  === \ False, \ ans, \ 
            Pad[ans]]]\)\(\n\)\(\n\)\(\t\ \ \ \ \ \)\),
      FontFamily->"Courier New",
      FontSize->16], "\n", 
    StyleBox[\(\(NormalDecomposition[matrix_?MatrixQ]\  := \ \n\t
        Module[{factors}, \n\t\tfactors\  = \ 
            ElementaryDecomposition[matrix, \ Padded\  -> \ True]; \n\t\t{\ 
            Dot\  @@ \ factors[\([1]\)], \ factors[\([2, \ 1]\)], \ 
            Dot\  @@ \ factors[\([3]\)]\ }\ ]\)\(\n\)\(\t\t\)\),
      FontFamily->"Courier New",
      FontSize->16], "\n", 
    StyleBox[\(\(NormalDecomposition[matrix_?MatrixQ, \ t_]\  := \ \n\t
        Module[{factors}, \n\t\tfactors\  = \ 
            ElementaryDecomposition[matrix, \ t, \ 
              Padded\  -> \ True]; \n\t\t{\ Dot\  @@ \ factors[\([1]\)], \ 
            factors[\([2, \ 1]\)], \ 
            Dot\  @@ \ factors[\([3]\)]\ }\ ]\)\(\n\)\(\n\)\),
      FontFamily->"Courier New",
      FontSize->16], "\n", 
    StyleBox[\(\(InvariantFactors[matrix_?MatrixQ]\  := \n\t
        Module[{mat}, \n\t\tmat\  = \ \(ElementaryDecomposition[
                matrix]\)[\([2, \ 1]\)]; \n\t\tTable[
            mat[\([i, i]\)], \ {i, \ 1, \ 
              Min[Dimensions[matrix]]}]]\)\(\n\)\),
      FontFamily->"Courier New",
      FontSize->16], "\n", 
    StyleBox[\(\(InvariantFactors[matrix_?MatrixQ, \ t_]\  := \n\t
        Module[{mat}, \n\t\tmat\  = \ \(ElementaryDecomposition[matrix, \ 
                t]\)[\([2, \ 1]\)]; \n\t\tTable[
            mat[\([i, i]\)], \ {i, \ 1, \ 
              Min[Dimensions[
                  matrix]]}]]\)\(\n\)\(\n\)\(\n\)\(\n\)\(\n\)\( (*Elementary\ \
Matrix\ Padding*) \)\(\n\)\),
      FontFamily->"Courier New",
      FontSize->16], "\n", 
    StyleBox[\(\(SetAttributes[Pad, \ {Listable}];\)\(\n\)\),
      FontFamily->"Courier New",
      FontSize->16], "\n", 
    StyleBox[\(\(Pad[
          PermutationMatrix[i_, \ j_, \ n_]]\  := \ \n\t\(IdentityMatrix[
            n]\)[\([\ 
            Range[n]\  /. \ {i\  -> \ j, \ j\  -> \ i}\ ]\)]\)\(\n\)\),
      FontFamily->"Courier New",
      FontSize->16], "\n", 
    StyleBox[\(\(Pad[ElementaryMatrix[i_, \ j_, \ a_, \ n_]]\  := \ \n\t
        ReplacePart[IdentityMatrix[n], \ a, \ {i, \ j}]\)\(\n\)\),
      FontFamily->"Courier New",
      FontSize->16], "\n", 
    StyleBox[\(\(Pad[n_]\  := \ 
        n\)\(\n\)\(\t\)\(\n\)\(\n\)\( (*Testing*) \)\(\n\)\),
      FontFamily->"Courier New",
      FontSize->16], "\n", 
    StyleBox[\(\(CheckIt[factors_]\  := \ \ \n\t
        With[{fac\  = \ 
              Pad[factors]}, \ \n\t\t\((Dot\  @@ \ fac[\([1]\)])\)\  . \ 
            fac[\([2, 1]\)]\  . \ \((Dot\  @@ \ 
                fac[\([3]\)])\)]\)\(\n\)\(\n\)\(\n\)\(\n\)\(\n\)\( (*Formats\
*) \)\(\n\)\(\n\)\),
      FontFamily->"Courier New",
      FontSize->16], "\n", 
    StyleBox[\(\(Format[PermutationMatrix[i_, j_, n_]]\  := \ 
        Subscripted["\<p\>"[i, j]]\)\(\n\)\),
      FontFamily->"Courier New",
      FontSize->16], "\n", 
    StyleBox[\(\(Format[ElementaryMatrix[i_, j_, a_, \ n_]]\  := \ 
        Subscripted["\<e\>"[i, \ j, \ a], \ {1, \ 
            2}]\)\(\n\)\(\n\)\(\n\)\(\n\)\),
      FontFamily->"Courier New",
      FontSize->16], "\n", 
    StyleBox[\(\(End[]\)\(\ \)\( (*`private`*) \)\(\n\)\(\n\)\),
      FontFamily->"Courier New",
      FontSize->16], "\n", 
    StyleBox[\(\(Protect[ElementaryDecomposition, \ NormalDecomposition, \ 
        InvariantFactors]\)\(\n\)\(\n\)\),
      FontFamily->"Courier New",
      FontSize->16], "\n", 
    StyleBox[\(\(EndPackage[]\)\(\ \)\( (*ElementaryDecompositions`*) \)\(\n\
\)\(\n\)\(\n\)\(\n\)\( \
(**********************************************************) \)\(\n\)\( \
(*\(\(:\)\(Extras : \n\n
              MinimumPolynomial[A_?MatrixQ, \ t_Symbol]\)\)\  := \ \n\t
          Module[{m, \ p, \ c}, \n\t\tm\  = \ Length[A]; \n\t\tp\  = \ 
                Last[\ \ InvariantFactors[t\ IdentityMatrix[m]\  - \ A, \ 
                    t]\ \ ]; \n\t\tp/Last[CoefficientList[p, \ t]]]\  /; \ 
            SameQ[Dimensions[A]]\n*) \)\(\n\)\(\n\)\),
      FontFamily->"Courier New",
      FontSize->16]}], "Input",
  InitializationCell->True]
},
FrontEndVersion->"4.0 for Macintosh",
ScreenRectangle->{{0, 1024}, {0, 748}},
AutoGeneratedPackage->Automatic,
WindowSize->{520, 626},
WindowMargins->{{17, Automatic}, {Automatic, 41}},
MacintoshSystemPageSetup->"\<\
00<0001804P000000`d26_oQon@3:`8g0dL5N`?P0080001804P000000]P2:001
0000I00000400`<30?l00BL?00400@00000000000000060801T1T00000000000
00000000004000000000000000000000\>"
]


(***********************************************************************
Cached data follows.  If you edit this Notebook file directly, not using
Mathematica, you must remove the line containing CacheID at the top of 
the file.  The cache data will then be recreated when you save this file 
from within Mathematica.
***********************************************************************)

(*CellTagsOutline
CellTagsIndex->{}
*)

(*CellTagsIndex
CellTagsIndex->{}
*)

(*NotebookFileOutline
Notebook[{
Cell[1717, 49, 20809, 403, 8322, "Input",
  InitializationCell->True]
}
]
*)




(***********************************************************************
End of Mathematica Notebook file.
***********************************************************************)

